#ifndef IMAGE_H
#define IMAGE_H

#include <QGLWidget>
#include "imagewidgetbase.h"

class QImage;
class ExternalScaleWidget;
class ImageMouseEventInterface;

class Image : public QWidget , public ImageWidgetBase
{
    Q_OBJECT
    Q_PROPERTY(bool gridEnabled READ gridEnabled WRITE setGridEnabled)
    Q_PROPERTY(QColor gridColor READ gridColor WRITE setGridColor)
    Q_PROPERTY(QColor gridTextColor READ gridTextColor WRITE setGridTextColor)

public:
    explicit Image(QWidget *parent = 0);

    /* move all non Q_PROPERTY related methods into ImageWidgetBase.
     * Leave setImage because it calls update(), thus avoiding putting code
     * like mWidget->update() in the subclass
     */
    
    bool gridEnabled() const { return mGridEnabled; }

    QColor gridColor() const { return mGridColor; }

    QColor gridTextColor() const { return mGridTextColor; }

    void setImage(const QImage& image);

signals:
    
public slots:
    void setGridEnabled(bool en) { mGridEnabled = en; update(); }

    void setGridTextColor(const QColor& c) { mGridTextColor = c; update(); }

    void setGridColor(const QColor &c) { mGridColor = c; update(); }

    void setErrorImageEnabled(bool en);

protected:

    void paintEvent(QPaintEvent *);

    void mousePressEvent(QMouseEvent *ev);

    void mouseMoveEvent(QMouseEvent *ev);

    void mouseReleaseEvent(QMouseEvent *ev);

private:

    
};

#endif // IMAGE_H
