#ifndef IMAGEWIDGETBASE_H
#define IMAGEWIDGETBASE_H

#include <QColor>
#include <QPoint>
#include <QImage>

class ImageMouseEventInterface;
class ExternalScaleWidget;
class QPaintEvent;
class QPaintDevice;
class QWidget;
class QMouseEvent;

class ImageWidgetBase
{
public:
    ImageWidgetBase(QWidget *widget, bool isOpenGL);

    virtual ~ImageWidgetBase();

    void setImageMouseEventInterface(ImageMouseEventInterface* ifa);

    void initGridFromScales(ExternalScaleWidget *xScale, ExternalScaleWidget *yScale);

    void setError(bool error) { mError = error ; }

    void setErrorMessage(const QString& msg) { mErrorMessage = msg; }

    void setZoom(int n) { mZoom = n; }

    bool isOpenGL() const { return mIsOpenGL; }

    void setErrorImageEnabled(bool en);

protected:

    void mouseMove(QMouseEvent *e);

    void mousePress(QMouseEvent *e);

    void mouseRelease(QMouseEvent *e);

    void paint(QPaintEvent *e, QWidget *paintDevice);

    bool mGridEnabled;

    int mDiv, mZoom;

    double mMin, mMax;

    ExternalScaleWidget* mXScale, *mYScale;

    QColor mGridColor, mGridTextColor;

    bool mError;

    QString mErrorMessage;

    ImageMouseEventInterface *mMouseEventIf;

    bool mLeftButton;

    QPoint mP1, mP2;

    QImage mImage, mErrorImage;

private:
    QWidget *mWidget;

    bool mIsOpenGL;
};

#endif // IMAGEWIDGETBASE_H
