#ifndef CIRCULARDATAMANAGER_H
#define CIRCULARDATAMANAGER_H

#include <QObject>
#include <QColor>
#include <QList>
#include <QVector>
#include <com_proxy_reader.h>

class TVariant;
class SceneCurve;
class PlotSceneWidget;
class LinePainter;
class TangoConfigurationParameters;

class CircularDataManager : public QObject, public QTangoComProxyReader
{
   Q_OBJECT

    Q_PROPERTY(QColor oldestCurveColor READ oldestCurveColor WRITE setOldestCurveColor)
    Q_PROPERTY(QColor newestCurveColor READ newestCurveColor WRITE setNewestCurveColor)
    Q_PROPERTY(int bufferSize READ bufferSize WRITE setBufferSize)
    Q_PROPERTY(int period READ period WRITE setPeriod)
    Q_PROPERTY(int alphaChannel READ alphaChannel WRITE setAlphaChannel)

public:
    explicit CircularDataManager(PlotSceneWidget *plot);
    
    int bufferSize() const { return mBufferSize; }

    QColor newestCurveColor() const { return mNewestColor; }

    QColor oldestCurveColor() const { return mOldestColor; }

    QString source() const { return mSource; }

    QList<QColor> colorList() const { return mColorMap; }

    bool manualColorsEnabled() const { return mManualColors; }

    int alphaChannel() const { return mAlphaChannel; }

signals:
    void periodChanged(int millis);
    
public slots:

    void setNewestCurveColor(const QColor &c) { mNewestColor = c; initColorMap(); }

    void setOldestCurveColor(const QColor &c) { mOldestColor = c; initColorMap(); }

    /** \brief sets a custom list of colors to be used
     *
     * Automatically sets the manualColorsEnabled property to true
     */
    void setColorList(const QList<QColor> colors);

    void setManualColorsEnabled(bool manual) {mManualColors = manual; initColorMap(); }

    void setBufferSize(int s);

    void setPeriod(int p);

    void setAlphaChannel(int a) { mAlphaChannel = a; }

protected slots:

    void refresh(const TVariant &v);

    void attributeConfigured(const TangoConfigurationParameters *cp);

    void initColorMap();

    void setToolTip(const QString&);

private:
    int mBufferSize;

    QColor mNewestColor, mOldestColor;

    QList<SceneCurve *> mCurves;

    QList<QColor > mColorMap;

    QVector<double> mXData;

    PlotSceneWidget *mPlot;

    QString mSource;

    unsigned mNewestIndex;

    bool mManualColors;

    int mAlphaChannel;
    
};

#endif // CIRCULARDATAMANAGER_H
