#include "profiles.h"
#include "circulardatamanager.h"
#include <macros.h>
#include <QTimer>
#include <scenecurve.h>
#include <curveitem.h>
#include <linepainter.h>
#include <markeritem.h>
#include <QCoreApplication>
#include <elettracolors.h>
#include <QScrollBar>

Profiles::Profiles(QWidget *parent) :
    QWidget(parent)
{
    ui.setupUi(this);
    setAttribute(Qt::WA_DeleteOnClose, true);
    this->setWindowFlags(Qt::Window);

    ui.horProfile->setSettingsKey(qApp->applicationName());
    ui.verProfile->setSettingsKey(qApp->applicationName());

    /* x axis will have at most bufsiz points */
    ui.horProfile->xScaleItem()->setBounds(0, 1000);
    ui.verProfile->xScaleItem()->setBounds(0, 1000);
    ui.horProfile->yScaleItem()->setBounds(0, 1000);
    ui.verProfile->yScaleItem()->setBounds(0, 1000);

    /* load from configuration files (QSettings managed) */
    ui.horProfile->loadConfigurationProperties();
    ui.verProfile->loadConfigurationProperties();

    ui.hxScale->setOrientation(ScaleItem::Horizontal);
    ui.vxScale->setOrientation(ScaleItem::Horizontal);

    ui.hyScale->setOrientation(ScaleItem::Vertical);
    ui.vyScale->setOrientation(ScaleItem::Vertical);

//    connect(ui.horProfile->horizontalScrollBar(), SIGNAL(valueChanged(int)), ui.hxScale,
//            SLOT(scrollChanged(int)));
//    connect(ui.horProfile->verticalScrollBar(), SIGNAL(valueChanged(int)), ui.hyScale,
//            SLOT(scrollChanged(int)));

//    connect(ui.verProfile->horizontalScrollBar(), SIGNAL(valueChanged(int)), ui.vxScale,
//            SLOT(scrollChanged(int)));
//    connect(ui.verProfile->verticalScrollBar(), SIGNAL(valueChanged(int)), ui.vyScale,
//            SLOT(scrollChanged(int)));

//    connect(ui.horProfile, SIGNAL(plotAreaChanged(QSizeF)), ui.hxScale, SLOT(areaChanged(QSizeF)));
//    connect(ui.horProfile, SIGNAL(plotAreaChanged(QSizeF)), ui.hyScale, SLOT(areaChanged(QSizeF)));

//    connect(ui.verProfile, SIGNAL(plotAreaChanged(QSizeF)), ui.vxScale, SLOT(areaChanged(QSizeF)));
//    connect(ui.verProfile, SIGNAL(plotAreaChanged(QSizeF)), ui.vyScale, SLOT(areaChanged(QSizeF)));

    ui.horProfile->xScaleItem()->installAxisChangeListener(ui.hxScale);
    ui.horProfile->yScaleItem()->installAxisChangeListener(ui.hyScale);

    ui.verProfile->xScaleItem()->installAxisChangeListener(ui.vxScale);
    ui.verProfile->yScaleItem()->installAxisChangeListener(ui.vyScale);

    ui.horProfile->installPlotGeometryChangeListener(ui.hxScale);
    ui.horProfile->installPlotGeometryChangeListener(ui.hyScale);
    ui.verProfile->installPlotGeometryChangeListener(ui.vxScale);
    ui.verProfile->installPlotGeometryChangeListener(ui.vyScale);

    ui.horProfile->addConfigurableObjects("Horizontal X scale", ui.hxScale);
    ui.horProfile->addConfigurableObjects("Horizontal Y scale", ui.hyScale);
    ui.verProfile->addConfigurableObjects("Vertical X scale", ui.vxScale);
    ui.verProfile->addConfigurableObjects("Vertical y scale", ui.vyScale);

    foreach(ExternalScaleWidget *extScale, findChildren<ExternalScaleWidget *>())
    {
        extScale->setSyncAxisLabelsWithPlot(true);
        extScale->setSyncAxisTicksWithPlot(true);
    }

    ui.hyScale->setOriginPercentage(0.5);
    ui.hxScale->setOriginPercentage(0.5);
    ui.vyScale->setOriginPercentage(0.5);
    ui.vxScale->setOriginPercentage(0.5);

    ui.hxScale->setAlignment(Qt::AlignHCenter);
    ui.vxScale->setAlignment(Qt::AlignHCenter);
    ui.hyScale->setAlignment(Qt::AlignVCenter);
    ui.vyScale->setAlignment(Qt::AlignVCenter);

//    ui.hxScale->setVisible(false);
//        ui.vxScale->setVisible(false);
//        ui.hyScale->setVisible(false);
//        ui.vyScale->setVisible(false);

    QTimer::singleShot(100, this, SLOT(setup()));
}

Profiles::~Profiles()
{
    printf("\e[1;31m deleting Profiles\e[0m\n");
    delete ui.horProfile->findChild<CircularDataManager *>();
    delete ui.verProfile->findChild<CircularDataManager *>();
}

void Profiles::setup()
{
    CircularDataManager *hCircularDataManager = new CircularDataManager(ui.horProfile);
    CircularDataManager *vCircularDataManager = new CircularDataManager(ui.verProfile);
    hCircularDataManager->setObjectName("HCircularDataManager");
    hCircularDataManager->setPeriod(100);
    hCircularDataManager->setNewestCurveColor(KVERYDARKBLUE);
    hCircularDataManager->setOldestCurveColor(KGRAY);
    hCircularDataManager->setSource("$1/HorProfile");

    vCircularDataManager->setObjectName("VCircularDataManager");
    vCircularDataManager->setPeriod(100);
    vCircularDataManager->setSource("$1/VerProfile");
    vCircularDataManager->setNewestCurveColor(Qt::black);
    vCircularDataManager->setOldestCurveColor(KGRAY);


    MarkerItem* marker = new MarkerItem(0);
    ui.horProfile->installMouseEventListener(marker);
    ui.horProfile->scene()->addItem(marker);
    connect(ui.horProfile, SIGNAL(curveAboutToBeRemoved(SceneCurve *)), marker,
            SLOT(removeCurve(SceneCurve *)));

    MarkerItem* vmarker = new MarkerItem(0);
    ui.verProfile->installMouseEventListener(vmarker);
    ui.verProfile->scene()->addItem(vmarker);
    connect(ui.verProfile, SIGNAL(curveAboutToBeRemoved(SceneCurve *)), marker,
            SLOT(removeCurve(SceneCurve *)));

    ui.horProfile->setPainterAntiAlias(false);
    ui.verProfile->setPainterAntiAlias(false);

    /* it seems that with the new version of QGraphicsPlot manual scene update is not necessary
     * ...
     */

    connect(hCircularDataManager, SIGNAL(periodChanged(int)), ui.horProfile, SLOT(setRefreshPeriod(int)));
    connect(vCircularDataManager, SIGNAL(periodChanged(int)), ui.verProfile, SLOT(setRefreshPeriod(int)));

    ui.horProfile->addConfigurableObjects("Horizontal Data Manager", hCircularDataManager);
    ui.verProfile->addConfigurableObjects("Vertical Data Manager", vCircularDataManager);
}

