/***************************************************************************
 *   $Id: spectrometer.h,v 1.15 2013-06-13 12:12:51 giacomo Exp $
 *                                                                         *
 *   Copyright (C) 2009 by Fabio Asnicar  fabio.asnicar@elettra.trieste.it *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef spectrometer_H
#define spectrometer_H

#include "ui_spectrometer.h"
#include "imagemouseeventinterface.h"
#include "simpleLog.h"
#include <tango.h>
#include <QWidget>
#include <QString>

#if QT_VERSION < 0x040800
#error "Please use at least Qt 4.8"
#endif

class Image;
class ImageGL;

/** \brief Spectrometer class
 *
 * See spectrometer_colormap.cpp file for a couple of methods related to the color map
 *
 */
class Spectrometer: public QWidget, public ImageMouseEventInterface
{

    Q_OBJECT

public:
    Spectrometer(QWidget * =NULL);
    ~Spectrometer();

public slots:
    void processingEnable();

    /* in spectrometer_colormap.cpp
     *
     */
    void setColorMap(bool);

    void save();

    void snapshot();

    /* zoom related methods */
    void zoomIn();   
    void zoomOut();   
    void zoomOriginal();   
    void zoomBestFit();

    void setPlotLevel(int);

    void updatePlotLevel(int);

    void setDelay(int);

    void measuringEnable(bool);

    void showInfo();

protected slots:

    void imageDepthChanged(int dpt);

    void toggleOpenGLWidget(bool openGL);

    void showProfiles();

//    void calculateWavelength(const TVariant& spectrumPeakPos);

    void updateScales();

    void oYagOriginPropertyReady(const QString &, const QStringList &);


signals:
    void imageSizeChanged(const QSizeF& newSize);

private:

    QWidget *getImageWidget() const;

    void initImageWidget();

    void generateColorMap();

    void showImage();

    void setZoom(int);

    float normalize(float, char);

    QString getFileName();

    void initMatrix(int,int);

    void initWidgets();

    /** \brief creates the widget that draws the image. A normal QWidget or a QGLWidget
     */
    void createImageWidget();

    /** \brief fills in the matrix starting from the 16 bit deep Tango Image.
     *
     * The tango data type is unsigned short. This method receives the
     * spectrum as it is from the tango read. vector2image16 and vector2image8
     * were introduced in order to save some for cycles over the vector
     */
    void vector2image16(const vector<unsigned short> &v);

    /** \brief fills in the matrix starting from the Tango Image at 8 bit depth
     *
     * @see vector2image16
     *
     */
    void vector2image8(const vector<unsigned char> &v);

    short imageDepth;
    double mSignHor;
    double mSignVer;

    QSize lastImageSize;

    int delay;
    int zoom, zoomStep;
    int maxX, maxY, diffY, mNRows, mNCols;
    double mOYagOriginX;

    /* contains the data of the matrix */
    unsigned char* mImageMatrix;

    QVector<int> mRoi, mLastpoint;
    QImage *mImage;
    Image *mImageWidget;
    ImageGL *mImageGLWidget;
    QPoint mUpperBoundPoint, mLowerBoundPoint;
    QVector<QRgb> paletteBW, paletteColor1, paletteColor2, paletteColorJet, paletteColorHot, colorMap;
    QString tangodev;
    Tango::DeviceProxy *server, *tdp_scr;
    Ui::Spectrometer ui;

    /* to calculate wavelength */
    double mWavelenSpan, mPixel2Micron, mWavelen;

protected:

    virtual void imageMousePressEvent(const QPoint& pos, int button);
    virtual void imageMouseReleaseEvent(const QPoint& pos, int button);
    virtual void imageMouseMoveEvent(const QPoint& pos, int button);

    virtual bool event(QEvent *);

};

#endif

